from autofaq.cli.commands.clean import clean
from autofaq.cli.commands.embed import embed
from autofaq.cli.commands.extract import extract
from autofaq.cli.commands.init import init
from autofaq.cli.commands.keywords import keywords
from autofaq.cli.commands.render import render
from autofaq.cli.commands.search import search
